/*
  * Copyright (c) 1997, 1998 University of Michigan, Ann Arbor.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
  * provided that the above copyright notice and this paragraph are
  * duplicated in all such forms and that any documentation,
  * advertising materials, and other materials related to such
  * distribution and use acknowledge that the software was developed
  * by the University of Michigan, Ann Arbor. The name of the University
  * may not be used to endorse or promote products derived from this
  * software without specific prior written permission.
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
  * Author: Sugih Jamin (jamin@eecs.umich.edu)
  *         Eric Cronin (ecronin@eecs.umich.edu)
  *         Wenjie Wang (wenjiew@eecs.umich.edu)
  *
  * "@(#) $Id: _turnind.c,v 1.4 2000/08/09 01:15:32 ecronin Exp $ (UM)";
*/

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "turnin.h"

extern int errno;

/*
  * tind_recv: receive a buffer-full of data.
  *
  * Arg "buf" is where received data shall be
  * stored.  "Buf" is of "blen" size.
  *
  * In case of client side's error, an error message is
  * returned, which must be freed by the caller.
  * On server side's system error, terminate process.
  *
  * On success, a NULL is returned and
  * "blen" contains the actual length of data received.
*/
char *
_tind_recv(int td, char *buf, int *blen)
{
   int i;
   int err;
   char *bp;
   int len, left;
   char *errmsg;

   /* initialize variables */
   i = 0;
   errno = 0;
   bp = buf;
   left = *blen;
   errmsg = (char *) 0;

   /* poll socket for data every TIND_SLEEPUSEC
      in case client or connection is slow and
      data hasn't arrived.  But do this only
      TIND_MAXTRIES times so that buggy client
      won't put us in an infinite loop.
      Better implemented with select(). */
   do {
     len = recv(td, bp, left, 0);

     if (len < 0 && errno == EWOULDBLOCK) {
       i++;
       len = 1;
       errno = 0;
       usleep(TIND_SLEEPUSEC);
     } else {
       i = 0;
       bp += len;
       left -= len;
     }
   } while (len > 0 && left > 0 && i < TIND_MAXTRIES);

   /* check for error on receiving. */
   if (len < 0) {
     err = errno;
     errmsg = (char *) malloc(TIN_MLEN);
     tin_check_err(!errmsg, "malloc");
     sprintf(errmsg, "Error %d when receiving.\n", err);
   } else if (i >= TIND_MAXTRIES) {
     errmsg = (char *) malloc(TIN_MLEN);
     tin_check_err(!errmsg, "malloc");
     sprintf(errmsg, "Timed out waiting for transfer.\n");
   } else if (*blen == left) {
     errmsg = (char *) malloc(TIN_MLEN);
     tin_check_err(!errmsg, "malloc");
     sprintf(errmsg, "Received nothing.\n");
   } else {
     /* update blen to length of data actually received. */
     *blen -= left;
   }

   return(errmsg);
}
